-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local currencyInfo = {};
local currencyHidden =
{
  [483] = true, -- Conquest Arena Meta
  [484] = true, -- Conquest Rated BG Meta
  [692] = true, -- Conquest Random BG Meta
};

local tokenFrameHooked = false;
local tokenToggles     = {};

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S
-------------------------------------------------------------------------------

local currencyToggleOnClick;
local currencyToggleOnEnter;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get info for currencies
--
local function currencyGetInfo()
  -- Local Variables
  local name;
  local totalMax;
  local weeklyMax;

  -- Get the IDs of those currencies that have weekly and/or total limits
  for currencyId = 1, 2000 do
    name, _, _, _, weeklyMax, totalMax = GetCurrencyInfo(currencyId);
    if (((name or "") ~= "") and
        ((weeklyMax > 0) or (totalMax > 0)) and
        (currencyHidden[currencyId] ~= true)) then
      currencyInfo[name] = {id = currencyId};
    end
  end

  return;
end

--
-- Function to update the Token Frame scroll frame
--
local function currencyListUpdate()
  -- Initialise some variables
  local button;
  local buttons;
  local currencies = NigglesDBC.currencies;
  local currencyId;
  local isWarningEnabled = NigglesDBC.settings.WarningCurrency;
  local name;
  local numButtons;
  local offset = HybridScrollFrame_GetOffset(TokenFrameContainer);
  local toggle;
  local toggleShown;
  local toggleChecked;

  -- Update the toggle for each button
  buttons    = TokenFrameContainer.buttons;
  numButtons = ((buttons ~= null) and #(TokenFrameContainer.buttons) or 0);
  for buttonIdx = 1, numButtons do
    -- Initialise some variables
    button        = buttons[buttonIdx];
    toggleShown   = false;
    toggleChecked = false;

    -- Check the button is shown
    if (button:IsShown()) then
      -- Create the toggle, if required
      if ((isWarningEnabled) and (tokenToggles[buttonIdx] == nil)) then
        toggle = CreateFrame("CheckButton", nil, button,
          "NigglesToggleTemplate");
        toggle:SetSize(14, 14);
        toggle:SetPoint("CENTER");
        toggle:SetPoint("RIGHT", button, "RIGHT", -1, 0);
        toggle:SetScript("OnClick", currencyToggleOnClick);
        toggle:SetScript("OnEnter", currencyToggleOnEnter);
        tokenToggles[buttonIdx] = toggle;
      end
      toggle = tokenToggles[buttonIdx];

      -- Work out the correct state for the toggle
      if (not button.isHeader) then
        name = GetCurrencyListInfo(button.index);
        if (currencyInfo[name] ~= nil) then
          currencyId    = currencyInfo[name].id;
          toggleShown   = isWarningEnabled;
          toggleChecked = (currencies[currencyId] ~= false);
        end
      end

      -- Update the toggle
      if (toggle ~= nil) then
        toggle:SetShown(toggleShown);
        if (toggleShown) then
          toggle:SetChecked(toggleChecked);
          toggle.currencyId = currencyId;
        end
      end

      -- Update the button, including a fix for a Blizzard striping bug
      button.check:Hide();
      button.stripe:SetShown((buttonIdx+offset)%2 == 0);
    end
  end

  return;
end

--
-- Function for processing clicks on a currency toggle button
--
currencyToggleOnClick = function(self, mouseButton)
  -- Local Variables
  local currencyId;
  local name;

  -- Process the mouse click
  if ((mouseButton == "LeftButton") and (self.currencyId ~= nil)) then
    -- Update the state for the currency
    if (not self:GetChecked()) then
      NigglesDBC.currencies[self.currencyId] = false;
    else
      NigglesDBC.currencies[self.currencyId] = nil;
    end
  end

  return;
end

--
-- Function to display the tooltip for a Pet toggle button
--
currencyToggleOnEnter = function(toggle)
  -- Set the game tooltip
  GameTooltip:SetOwner(toggle, "ANCHOR_RIGHT");
  GameTooltip:SetMinimumWidth(100);
  GameTooltip:SetText(L["CurrencyTooltip"], NORMAL_FONT_COLOR.r,
    NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1, 1);
  GameTooltip:Show();

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to check if any currency is at or near its limit
--
function NigglesCurrencyCheck()
  -- Local Variables
  local _;
  local currencies = NigglesDBC.currencies;
  local string = "";
  local texture;
  local totalAmt;
  local totalMax;
  local weeklyAmt;
  local weeklyMax;
  local warningMask = NigglesDBC.settings.WarningCurrencyMask;

  -- Get currency info, if required
  if (next(currencyInfo) == nil) then
    currencyGetInfo();
  end

  -- Search for currencies at or near their maximum
  for name, current in pairs(currencyInfo) do
    -- Check the currency should be monitored
    if (currencies[current.id] ~= false) then
      -- Get info for the currency
      _, totalAmt, texture, weeklyAmt, weeklyMax, totalMax =
        GetCurrencyInfo(current.id);

      -- Check if the currency is more than 90% of its weekly or total maximum
      if (((bit.band(warningMask, 0x01) ~= 0) and
           (weeklyMax > 0) and
           (weeklyAmt > (weeklyMax*0.9))) or
          ((bit.band(warningMask, 0x02) ~= 0) and
           (totalMax > 0) and
           (totalAmt > (totalMax*0.9)))) then
        -- Check if the player has already been warned
        if (current.warned ~= true) then
          -- NOTE: The '|r' is done after the '\n' to overcome a Blizzard bug
          string = string.."\124T"..texture..
            ":0:0:0:-8\124t "..GetCurrencyLink(current.id).."\n|r";
          current.warned = true;
        end
      else
        current.warned = false;
      end
    end
  end

  -- Check if there are any warnings to be displayed
  if (string ~= "") then
    -- Display the warning
    NigglesWarningPopupInit(L["CurrencyMax"], nil, OKAY, nil, nil);
    NigglesWarningPopup.list:SetText(string);
    NigglesWarningPopup.list:Show();
    StaticPopupSpecial_Show(NigglesWarningPopup);
  end

  return;
end

--
-- Function to enable/disable Currency features
--
function NigglesCurrencyEnable()
  -- Hook the update function for the Token Frame's scroll frame, if required
  if (not tokenFrameHooked) then
    hooksecurefunc("TokenFrame_Update", currencyListUpdate);
    TokenFrameContainer.update = TokenFrame_Update;
    tokenFrameHooked = true;
  end

  -- Update the toggles in the Token Frame's scroll frame, if required
  if (TokenFrameContainer:IsVisible()) then
    currencyListUpdate();
  end

  return;
end
